require "thread"
require "logger"
# Add the thrift directory to the path since the generated code doesn't use
# relative requires.
$:.push File.expand_path(File.join(File.dirname(__FILE__), "thrift"))
require "latency_metric_constants"
require "latency_metric_service"

module CromwellApi
class Client

  TIMESTAMP_FORMAT = "%Y-%m-%d %H:%M:%S".freeze

  def initialize(product, env, cromwell_host = "127.0.0.1", options = {})
    @host = cromwell_host
    # TODO: validate args, normalize env values
    @sender_host = options.fetch :sender_host, Socket.gethostbyname(Socket.gethostname).first
    @product = product
    @env = env
    @backlog = Queue.new
    @batched_metrics = Queue.new
    @port = options.fetch :port, 9090
    @timeout = options.fetch :timeout, 5
    @batch_size = options.fetch :batch_size, 10
    start_metric_sender
  end

  def start_metric_sender
    Thread.new do
      loop do
        metrics = @batched_metrics.pop
        begin
          client.sendMetrics metrics
        rescue => e
          logger.error "CromwellApi: #{e.class} #{e}"
        end
      end
    end
  end

  def send_metric(service, ms)
    lm = CromwellThrift::LatencyMetric.new
    lm.timestamp = Time.now.utc.strftime TIMESTAMP_FORMAT
    lm.product = @product
    lm.host = @sender_host
    lm.metric = service
    lm.environment = @env
    lm.value = ms

    queue_metric lm
    nil
  end

  def send_metrics
    metrics = []
    metrics << @backlog.pop until @backlog.empty?
    @batched_metrics << metrics
  ensure
    @backlog.clear
  end

  def queue_metric(lm)
    @backlog << lm
    send_metrics if @backlog.size >= @batch_size
  end

  def client
    Thread.current[:thrift_client] ||= begin
      protocol = Thrift::BinaryProtocol.new transport
      CromwellThrift::LatencyMetricService::Client.new protocol
    end
    unless transport.open?
      transport.open
    end
    Thread.current[:thrift_client]
  end

  def transport
    Thread.current[:thrift_transport] ||= begin
      Thrift::FramedTransport.new Thrift::Socket.new(@host, @port, @timeout)
    end
  end

  def set_logger(logger)
    @logger = logger
  end

  def logger
    @logger ||= Logger.new $stdout
  end

end
end
